"""Setup wizard for first-time groknroll configuration."""

from groknroll.setup.wizard import run_setup, check_first_run, get_config_dir

__all__ = ["run_setup", "check_first_run", "get_config_dir"]
